@@ -4,13 +4,12 @@ use anyhow::{anyhow, Result};
 use clap::{Parser, Subcommand};
 use indicatif::ProgressBar;
 
-use super::progress_counter;
-use super::rustic_config::RusticConfig;
+use super::{progress_counter, Config};
 use crate::backend::{DecryptReadBackend, FileType};
 use crate::blob::{BlobType, Tree};
 use crate::id::Id;
 use crate::index::{IndexBackend, IndexedBackend};
-use crate::repofile::{SnapshotFile, SnapshotFilter};
+use crate::repofile::SnapshotFile;
 use crate::repository::OpenRepository;
 
 #[derive(Parser)]
@@ -43,15 +42,12 @@ struct IdOpt {
 
 #[derive(Parser)]
 struct TreeOpts {
-    #[clap(flatten, help_heading = "SNAPSHOT FILTER OPTIONS (when using latest)")]
-    filter: SnapshotFilter,
-
     /// Snapshot/path of the tree to display
     #[clap(value_name = "SNAPSHOT[:PATH]")]
     snap: String,
 }
 
-pub(super) fn execute(repo: OpenRepository, opts: Opts, config_file: RusticConfig) -> Result<()> {
+pub(super) fn execute(repo: OpenRepository, config: Config, opts: Opts) -> Result<()> {
     let be = &repo.dbe;
     match opts.command {
         Command::Config => cat_file(be, FileType::Config, IdOpt::default()),
@@ -61,7 +57,7 @@ pub(super) fn execute(repo: OpenRepository, opts: Opts, config_file: RusticConfi
         Command::TreeBlob(opt) => cat_blob(be, BlobType::Tree, opt),
         Command::DataBlob(opt) => cat_blob(be, BlobType::Data, opt),
         // special treatment for cating a tree within a snapshot
-        Command::Tree(opts) => cat_tree(be, opts, config_file),
+        Command::Tree(opts) => cat_tree(be, config, opts),
     }
 }
 
@@ -81,15 +77,14 @@ fn cat_blob(be: &impl DecryptReadBackend, tpe: BlobType, opt: IdOpt) -> Result<(
     Ok(())
 }
 
-fn cat_tree(
-    be: &impl DecryptReadBackend,
-    mut opts: TreeOpts,
-    config_file: RusticConfig,
-) -> Result<()> {
-    config_file.merge_into("snapshot-filter", &mut opts.filter)?;
-
+fn cat_tree(be: &impl DecryptReadBackend, config: Config, opts: TreeOpts) -> Result<()> {
     let (id, path) = opts.snap.split_once(':').unwrap_or((&opts.snap, ""));
-    let snap = SnapshotFile::from_str(be, id, |sn| sn.matches(&opts.filter), progress_counter(""))?;
+    let snap = SnapshotFile::from_str(
+        be,
+        id,
+        |sn| sn.matches(&config.snapshot_filter),
+        progress_counter(""),
+    )?;
     let index = IndexBackend::new(be, progress_counter(""))?;
     let node = Tree::node_from_path(&index, snap.tree, Path::new(path))?;
     let id = node.subtree.ok_or_else(|| anyhow!("{path} is no dir"))?;
