@@ -5,7 +5,7 @@ use std::{cmp::Ordering, fmt::Display};
 
 use anyhow::{anyhow, bail, Result};
 use chrono::{DateTime, Duration, Local};
-use clap::{AppSettings, Parser};
+use clap::Parser;
 use derivative::Derivative;
 use dunce::canonicalize;
 use gethostname::gethostname;
@@ -17,7 +17,7 @@ use path_dedot::ParseDot;
 use rhai::serde::to_dynamic;
 use rhai::{Dynamic, Engine, FnPtr, AST};
 use serde::{Deserialize, Serialize};
-use serde_with::{serde_as, DisplayFromStr};
+use serde_with::{serde_as, DeserializeFromStr, DisplayFromStr};
 
 use super::Id;
 use crate::backend::{DecryptReadBackend, FileType, RepoFile};
@@ -25,7 +25,6 @@ use crate::repository::parse_command;
 
 #[serde_as]
 #[derive(Clone, Default, Parser, Deserialize, Merge)]
-#[clap(global_setting(AppSettings::DeriveDisplayOrder))]
 #[serde(default, rename_all = "kebab-case", deny_unknown_fields)]
 pub struct SnapshotOptions {
     /// Label snapshot with given label
@@ -47,7 +46,7 @@ pub struct SnapshotOptions {
     description_from: Option<PathBuf>,
 
     /// Mark snapshot as uneraseable
-    #[clap(long, conflicts_with = "delete-after")]
+    #[clap(long, conflicts_with = "delete_after")]
     #[merge(strategy = merge::bool::overwrite_false)]
     delete_never: bool,
 
@@ -420,6 +419,7 @@ impl Ord for SnapshotFile {
     }
 }
 
+#[derive(Clone)]
 struct SnapshotFn(FnPtr, AST);
 impl FromStr for SnapshotFn {
     type Err = anyhow::Error;
@@ -440,7 +440,7 @@ impl SnapshotFn {
 }
 
 #[serde_as]
-#[derive(Default, Parser, Deserialize, Merge)]
+#[derive(Clone, Default, Parser, Deserialize, Merge)]
 #[serde(default, rename_all = "kebab-case", deny_unknown_fields)]
 pub struct SnapshotFilter {
     /// Hostname to filter (can be specified multiple times)
@@ -471,7 +471,7 @@ pub struct SnapshotFilter {
     filter_fn: Option<SnapshotFn>,
 }
 
-#[derive(Clone, Default, Deserialize)]
+#[derive(Clone, Default, DeserializeFromStr)]
 pub struct SnapshotGroupCriterion {
     hostname: bool,
     label: bool,
